/// Web Serial API
///
/// https://wicg.github.io/serial/
@JS('window')
@staticInterop
library serial;

import 'dart:js_util' as js_util;
import 'package:js/js.dart';

import 'callbacks.dart';
import '../manual.dart';
import 'all_bindings.dart';
/* deps: html
dom
streams */

///  Secure context: This feature is available only in secure
/// contexts (HTTPS), in some or all supporting browsers.
@JS()
@staticInterop
class Serial implements EventTarget {
  external Serial();
}

extension PropsSerial on Serial {
  EventHandlerNonNull? get onconnect => js_util.getProperty(this, 'onconnect');
  set onconnect(EventHandlerNonNull? newValue) {
    js_util.setProperty(this, 'onconnect', newValue);
  }

  EventHandlerNonNull? get ondisconnect =>
      js_util.getProperty(this, 'ondisconnect');
  set ondisconnect(EventHandlerNonNull? newValue) {
    js_util.setProperty(this, 'ondisconnect', newValue);
  }

  ///
  ///     Returns a [Promise] that resolves with an array of
  /// [SerialPort] objects representing serial ports connected to
  ///    the host which the origin has permission to access.
  ///
  ///
  /// var promise = Serial.getPorts();
  ///
  Iterable<Promise<SerialPort>> getPorts() =>
      js_util.callMethod(this, 'getPorts', []);

  ///  Returns a [Promise] that resolves with an instance of
  /// [SerialPort] representing the device chosen by the user or
  /// rejects if no device was selected.
  ///   This method must be called with user activation.
  ///
  /// var promise = Serial.requestPort([options]);
  ///
  Future<SerialPort> requestPort([SerialPortRequestOptions? options]) => js_util
      .promiseToFuture(js_util.callMethod(this, 'requestPort', [options]));
}

@anonymous
@JS()
@staticInterop
class SerialPortRequestOptions {
  external factory SerialPortRequestOptions(
      {Iterable<SerialPortFilter> filters});
}

extension PropsSerialPortRequestOptions on SerialPortRequestOptions {
  Iterable<SerialPortFilter> get filters =>
      js_util.getProperty(this, 'filters');
  set filters(Iterable<SerialPortFilter> newValue) {
    js_util.setProperty(this, 'filters', newValue);
  }
}

@anonymous
@JS()
@staticInterop
class SerialPortFilter {
  external factory SerialPortFilter({int usbVendorId, int usbProductId});
}

extension PropsSerialPortFilter on SerialPortFilter {
  int get usbVendorId => js_util.getProperty(this, 'usbVendorId');
  set usbVendorId(int newValue) {
    js_util.setProperty(this, 'usbVendorId', newValue);
  }

  int get usbProductId => js_util.getProperty(this, 'usbProductId');
  set usbProductId(int newValue) {
    js_util.setProperty(this, 'usbProductId', newValue);
  }
}

///  Secure context: This feature is available only in secure
/// contexts (HTTPS), in some or all supporting browsers.
@JS()
@staticInterop
class SerialPort implements EventTarget {
  external SerialPort();
}

extension PropsSerialPort on SerialPort {
  EventHandlerNonNull? get onconnect => js_util.getProperty(this, 'onconnect');
  set onconnect(EventHandlerNonNull? newValue) {
    js_util.setProperty(this, 'onconnect', newValue);
  }

  EventHandlerNonNull? get ondisconnect =>
      js_util.getProperty(this, 'ondisconnect');
  set ondisconnect(EventHandlerNonNull? newValue) {
    js_util.setProperty(this, 'ondisconnect', newValue);
  }

  ///  Returns a [ReadableStream] for receiving data from the device
  /// connected to the port.
  ///
  ReadableStream get readable => js_util.getProperty(this, 'readable');

  ///  Returns a [WritableStream] for sending data to the device
  /// connected to the port.
  ///
  WritableStream get writable => js_util.getProperty(this, 'writable');

  ///  Returns a [Promise] that resolves with an object containing
  /// properties of the port.
  ///
  /// var promise = SerialPort.getInfo();
  ///
  SerialPortInfo getInfo() => js_util.callMethod(this, 'getInfo', []);

  ///  Returns a [Promise] that resolves when the port is opened. By
  /// default the port is opened with 8 data bits, 1 stop bit and no
  /// parity checking.
  ///
  /// var promise = SerialPort.open(options);
  ///
  Future<Object> open(SerialOptions options) =>
      js_util.promiseToFuture(js_util.callMethod(this, 'open', [options]));

  ///  Sets control signals on the port and returns a [Promise] that
  /// resolves when they are set.
  ///
  /// var promise = SerialPort.setSignals(options);
  ///
  Future<Object> setSignals([SerialOutputSignals? signals]) => js_util
      .promiseToFuture(js_util.callMethod(this, 'setSignals', [signals]));

  ///  Returns a [Promise] that resolves with an object containing the
  /// current state of the port's control signals.
  ///
  /// var promise = SerialPort.getSignals();
  ///
  Future<SerialInputSignals> getSignals() =>
      js_util.promiseToFuture(js_util.callMethod(this, 'getSignals', []));

  /// Returns a [Promise] that resolves when the port closes.
  ///
  /// var promise = SerialPort.close();
  ///
  Future<Object> close() =>
      js_util.promiseToFuture(js_util.callMethod(this, 'close', []));
}

@anonymous
@JS()
@staticInterop
class SerialPortInfo {
  external factory SerialPortInfo({int usbVendorId, int usbProductId});
}

extension PropsSerialPortInfo on SerialPortInfo {
  int get usbVendorId => js_util.getProperty(this, 'usbVendorId');
  set usbVendorId(int newValue) {
    js_util.setProperty(this, 'usbVendorId', newValue);
  }

  int get usbProductId => js_util.getProperty(this, 'usbProductId');
  set usbProductId(int newValue) {
    js_util.setProperty(this, 'usbProductId', newValue);
  }
}

@anonymous
@JS()
@staticInterop
class SerialOptions {
  external factory SerialOptions(
      {int baudRate,
      int dataBits = 8,
      int? stopBits = 1,
      ParityType? parity = ParityType.none,
      int? bufferSize = 255,
      FlowControlType? flowControl = FlowControlType.none});
}

extension PropsSerialOptions on SerialOptions {
  int get baudRate => js_util.getProperty(this, 'baudRate');
  set baudRate(int newValue) {
    js_util.setProperty(this, 'baudRate', newValue);
  }

  int get dataBits => js_util.getProperty(this, 'dataBits');
  set dataBits(int newValue) {
    js_util.setProperty(this, 'dataBits', newValue);
  }

  int get stopBits => js_util.getProperty(this, 'stopBits');
  set stopBits(int newValue) {
    js_util.setProperty(this, 'stopBits', newValue);
  }

  ParityType get parity => js_util.getProperty(this, 'parity');
  set parity(ParityType newValue) {
    js_util.setProperty(this, 'parity', newValue);
  }

  int get bufferSize => js_util.getProperty(this, 'bufferSize');
  set bufferSize(int newValue) {
    js_util.setProperty(this, 'bufferSize', newValue);
  }

  FlowControlType get flowControl => js_util.getProperty(this, 'flowControl');
  set flowControl(FlowControlType newValue) {
    js_util.setProperty(this, 'flowControl', newValue);
  }
}

enum ParityType { none, even, odd }

enum FlowControlType { none, hardware }

@anonymous
@JS()
@staticInterop
class SerialOutputSignals {
  external factory SerialOutputSignals(
      {bool dataTerminalReady, bool requestToSend, bool mBreak});
}

extension PropsSerialOutputSignals on SerialOutputSignals {
  bool get dataTerminalReady => js_util.getProperty(this, 'dataTerminalReady');
  set dataTerminalReady(bool newValue) {
    js_util.setProperty(this, 'dataTerminalReady', newValue);
  }

  bool get requestToSend => js_util.getProperty(this, 'requestToSend');
  set requestToSend(bool newValue) {
    js_util.setProperty(this, 'requestToSend', newValue);
  }

  @JS('break')
  @staticInterop
  bool get mBreak => js_util.getProperty(this, 'break');
  set mBreak(bool newValue) {
    js_util.setProperty(this, 'break', newValue);
  }
}

@anonymous
@JS()
@staticInterop
class SerialInputSignals {
  external factory SerialInputSignals(
      {bool dataCarrierDetect,
      bool clearToSend,
      bool ringIndicator,
      bool dataSetReady});
}

extension PropsSerialInputSignals on SerialInputSignals {
  bool get dataCarrierDetect => js_util.getProperty(this, 'dataCarrierDetect');
  set dataCarrierDetect(bool newValue) {
    js_util.setProperty(this, 'dataCarrierDetect', newValue);
  }

  bool get clearToSend => js_util.getProperty(this, 'clearToSend');
  set clearToSend(bool newValue) {
    js_util.setProperty(this, 'clearToSend', newValue);
  }

  bool get ringIndicator => js_util.getProperty(this, 'ringIndicator');
  set ringIndicator(bool newValue) {
    js_util.setProperty(this, 'ringIndicator', newValue);
  }

  bool get dataSetReady => js_util.getProperty(this, 'dataSetReady');
  set dataSetReady(bool newValue) {
    js_util.setProperty(this, 'dataSetReady', newValue);
  }
}
